@@ -26,6 +26,7 @@ import (
 	"github.com/gohugoio/hugo/identity"
 	"github.com/gohugoio/hugo/resources/internal"
 
+	"github.com/gohugoio/hugo/common/hashing"
 	"github.com/gohugoio/hugo/common/herrors"
 	"github.com/gohugoio/hugo/common/paths"
 
@@ -65,6 +66,9 @@ type ResourceSourceDescriptor struct {
 	// The name of the resource as it was read from the source.
 	NameOriginal string
 
+	// The title of the resource.
+	Title string
+
 	// Any base paths prepended to the target path. This will also typically be the
 	// language code, but setting it here means that it should not have any effect on
 	// the permalink.
@@ -79,6 +83,9 @@ type ResourceSourceDescriptor struct {
 	// The Data to associate with this resource.
 	Data map[string]any
 
+	// The Params to associate with this resource.
+	Params maps.Params
+
 	// Delay publishing until either Permalink or RelPermalink is called. Maybe never.
 	LazyPublish bool
 
@@ -107,8 +114,12 @@ func (fd *ResourceSourceDescriptor) init(r *Spec) error {
 		panic(errors.New("RelPath is empty"))
 	}
 
+	if fd.Params == nil {
+		fd.Params = make(maps.Params)
+	}
+
 	if fd.Path == nil {
-		fd.Path = paths.Parse("", fd.TargetPath)
+		fd.Path = r.Cfg.PathParser().Parse("", fd.TargetPath)
 	}
 
 	if fd.TargetPath == "" {
@@ -143,6 +154,10 @@ func (fd *ResourceSourceDescriptor) init(r *Spec) error {
 		fd.NameOriginal = fd.NameNormalized
 	}
 
+	if fd.Title == "" {
+		fd.Title = fd.NameOriginal
+	}
+
 	mediaType := fd.MediaType
 	if mediaType.IsZero() {
 		ext := fd.Path.Ext()
@@ -293,7 +308,7 @@ type fileInfo interface {
 }
 
 type hashProvider interface {
-	hash() string
+	hash() uint64
 }
 
 var _ resource.StaleInfo = (*StaleValue[any])(nil)
@@ -389,7 +404,7 @@ func (l *genericResource) size() int64 {
 	return l.h.size
 }
 
-func (l *genericResource) hash() string {
+func (l *genericResource) hash() uint64 {
 	if err := l.h.init(l); err != nil {
 		panic(err)
 	}
@@ -614,7 +629,7 @@ type targetPather interface {
 }
 
 type resourceHash struct {
-	value    string
+	value    uint64
 	size     int64
 	initOnce sync.Once
 }
@@ -622,7 +637,7 @@ type resourceHash struct {
 func (r *resourceHash) init(l hugio.ReadSeekCloserProvider) error {
 	var initErr error
 	r.initOnce.Do(func() {
-		var hash string
+		var hash uint64
 		var size int64
 		f, err := l.ReadSeekCloser()
 		if err != nil {
@@ -630,7 +645,7 @@ func (r *resourceHash) init(l hugio.ReadSeekCloserProvider) error {
 			return
 		}
 		defer f.Close()
-		hash, size, err = helpers.MD5FromReaderFast(f)
+		hash, size, err = hashImage(f)
 		if err != nil {
 			initErr = fmt.Errorf("failed to calculate hash: %w", err)
 			return
@@ -641,3 +656,7 @@ func (r *resourceHash) init(l hugio.ReadSeekCloserProvider) error {
 
 	return initErr
 }
+
+func hashImage(r io.ReadSeeker) (uint64, int64, error) {
+	return hashing.XXHashFromReader(r)
+}
